@@ -25,11 +25,20 @@
 
 #include <string.h>
 
-#include <gtk/gtk.h>
+#include <glib/glist.h>
 #include <gst/propertyprobe/propertyprobe.h>
 
 #include "applet.h"
 
+static gint
+sort_by_rank (GstElement * a, GstElement * b)
+{
+#define gst_element_rank(x) \
+  gst_plugin_feature_get_rank (GST_PLUGIN_FEATURE ( \
+      gst_element_get_factory (x)))
+  return gst_element_rank (b) - gst_element_rank (a);
+}
+
 /*
  * Probe for mixer elements. Set up GList * with elements,
  * where each element has a GObject data node set of the
@@ -128,6 +137,7 @@ create_mixer_collection (void)
       num++;
 
       /* and recreate this object, since we give it to the mixer */
+      g_free (title);
       title = g_strdup_printf (gettext("Unknown Volume Control %d"), num);
       element = gst_element_factory_create (factory, title);
     }
@@ -140,7 +150,7 @@ next:
     g_free (title);
   }
 
-  return collection;
+  return g_list_sort (collection, (GCompareFunc) sort_by_rank);
 }
 
 static gboolean
@@ -157,7 +167,9 @@ gnome_volume_applet_factory (PanelApplet *applet,
   }
 
   elements = create_mixer_collection ();
-  return gnome_volume_applet_setup (GNOME_VOLUME_APPLET (applet), elements);
+  gnome_volume_applet_setup (GNOME_VOLUME_APPLET (applet), elements);
+
+  return TRUE;
 }
 
 PANEL_APPLET_BONOBO_FACTORY (
